@@ -54,7 +54,7 @@ int main(int argc, char const **argv) {
   // See set_level function for levels.
   sopt::logging::set_level("debug");
   SOPT_HIGH_LOG("Read input file {}", input);
-  Image const image = sopt::notinstalled::read_standard_tiff(input);
+  Image const image = sopt::tools::read_standard_tiff(input);
   SOPT_HIGH_LOG("Image size: {} x {} = {}", image.cols(), image.rows(), image.size());
 
   SOPT_HIGH_LOG("Initializing sensing operator");
