@@ -6,6 +6,7 @@
 
 #ifdef SOPT_MPI
 
+#include <algorithm> // for std::copy
 #include <iostream>
 #include <memory>
 #include <mpi.h>
@@ -19,8 +20,7 @@
 #include <cxxabi.h>
 #include <typeinfo>
 
-namespace sopt {
-namespace mpi {
+namespace sopt::mpi {
 
 //! \brief A C++ wrapper for an mpi communicator
 //! \details All copies made of this communicator are shallow: they reference
@@ -47,7 +47,7 @@ class Communicator {
   static Communicator World() { return Communicator(MPI_COMM_WORLD); }
   static Communicator Self() { return Communicator(MPI_COMM_SELF); }
 
-  virtual ~Communicator(){};
+  virtual ~Communicator(){}
 
   //! The number of processes
   decltype(Impl::size) size() const { return impl ? impl->size : 1; }
@@ -71,12 +71,12 @@ class Communicator {
   void abort(const std::string &reason) const;
 
   //! Helper function for reducing
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, T>::type all_reduce(T const &value,
                                                                             MPI_Op operation) const;
 
   //! In-place reduction over an image
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value>::type all_reduce(Matrix<T> &image,
                                                                          MPI_Op operation) const {
     if (size() == 1) return;
@@ -84,7 +84,7 @@ class Communicator {
     MPI_Allreduce(MPI_IN_PLACE, image.data(), image.size(), registered_type(T(0)), operation,
                   **this);
   }
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value>::type all_reduce(Image<T> &image,
                                                                          MPI_Op operation) const {
     if (size() == 1) return;
@@ -92,7 +92,7 @@ class Communicator {
     MPI_Allreduce(MPI_IN_PLACE, image.data(), image.size(), registered_type(T(0)), operation,
                   **this);
   }
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value>::type all_reduce(Vector<T> &image,
                                                                          MPI_Op operation) const {
     if (size() == 1) return;
@@ -102,16 +102,16 @@ class Communicator {
   }
 
   //! Helper function for reducing through sum
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, T>::type all_sum_all(T const &value) const {
     return all_reduce(value, MPI_SUM);
   }
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<typename T::Scalar>::value>::type all_sum_all(
       T &image) const {
     all_reduce(image, MPI_SUM);
   }
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<typename T::Scalar>::value, T>::type all_sum_all(
       T const &image) const {
     T result(image);
@@ -120,25 +120,25 @@ class Communicator {
   }
 
   //! Broadcasts object
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, T>::type broadcast(
       T const &value, t_uint const root = root_id()) const;
   //! Receive broadcast object
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, T>::type broadcast(
       t_uint const root = root_id()) const;
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<typename T::Scalar>::value, T>::type broadcast(
       T const &vec, t_uint const root = root_id()) const;
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<typename T::Scalar>::value, T>::type broadcast(
       t_uint const root = root_id()) const;
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<typename T::value_type>::value and
                               not std::is_base_of<Eigen::EigenBase<T>, T>::value,
                           T>::type
   broadcast(T const &vec, t_uint const root = root_id()) const;
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<typename T::value_type>::value and
                               not std::is_base_of<Eigen::EigenBase<T>, T>::value,
                           T>::type
@@ -146,61 +146,69 @@ class Communicator {
   std::string broadcast(std::string const &input, t_uint const root = root_id()) const;
 
   //! Scatter one object per proc
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, T>::type scatter_one(
       std::vector<T> const &values, t_uint const root = root_id()) const;
   //! Receive scattered objects
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, T>::type scatter_one(
       t_uint const root = root_id()) const;
 
   //! Scatter
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type scatterv(
       Vector<T> const &vec, std::vector<t_int> const &sizes, t_uint const root = root_id()) const;
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type scatterv(
       t_int local_size, t_uint const root = root_id()) const;
 
   // Gather one object per proc
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, std::vector<T>>::type gather(
       T const value, t_uint const root = root_id()) const;
 
   //! Gather
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type gather(
       Vector<T> const &vec, std::vector<t_int> const &sizes, t_uint const root = root_id()) const {
     return gather_<Vector<T>, T>(vec, sizes, root);
   }
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type gather(
       Vector<T> const &vec, t_uint const root = root_id()) const {
     return gather_<Vector<T>, T>(vec, root);
   }
 
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, std::set<T>>::type gather(
       std::set<T> const &set, std::vector<t_int> const &sizes, t_uint const root = root_id()) const;
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, std::set<T>>::type gather(
       std::set<T> const &vec, t_uint const root = root_id()) const;
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, std::vector<T>>::type gather(
       std::vector<T> const &vec, std::vector<t_int> const &sizes,
       t_uint const root = root_id()) const {
     return gather_<std::vector<T>, T>(vec, sizes, root);
   }
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, std::vector<T>>::type gather(
       std::vector<T> const &vec, t_uint const root = root_id()) const {
     return gather_<std::vector<T>, T>(vec, root);
   }
   //! All to all
-  template <class T>
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type all_to_allv(
       const Vector<T> &vec, std::vector<t_int> const &send_sizes) const;
-  template <class T>
+  template <typename T>
+  typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type all_to_allv(
+      const Vector<T> &vec, std::vector<t_int> const &send_sizes,
+      std::vector<t_int> const &rec_sizes) const;
+  template <typename T>
+  typename std::enable_if<is_registered_type<T>::value, std::vector<T>>::type all_to_allv(
+      const std::vector<T> &vec, std::vector<t_int> const &send_sizes,
+      std::vector<t_int> const &rec_sizes) const;
+  template <typename T>
   typename std::enable_if<is_registered_type<T>::value, std::vector<T>>::type all_to_allv(
       const std::vector<T> &vec, std::vector<t_int> const &send_sizes) const;
 
@@ -238,14 +246,14 @@ class Communicator {
   Communicator(MPI_Comm const &comm);
 
   //! Gather
-  template <class CONTAINER, class T>
+  template <typename CONTAINER, typename T>
   CONTAINER gather_(CONTAINER const &vec, std::vector<t_int> const &sizes,
                     t_uint const root = root_id()) const;
-  template <class CONTAINER, class T>
+  template <typename CONTAINER, typename T>
   CONTAINER gather_(CONTAINER const &vec, t_uint const root = root_id()) const;
 };
 
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<T>::value, T>::type Communicator::all_reduce(
     T const &value, MPI_Op operation) const {
   if (size() == 1) return value;
@@ -256,7 +264,7 @@ typename std::enable_if<is_registered_type<T>::value, T>::type Communicator::all
   return result;
 }
 
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<T>::value, T>::type Communicator::scatter_one(
     std::vector<T> const &values, t_uint const root) const {
   assert(root < size());
@@ -268,7 +276,7 @@ typename std::enable_if<is_registered_type<T>::value, T>::type Communicator::sca
   return result;
 }
 //! Receive scattered objects
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<T>::value, T>::type Communicator::scatter_one(
     t_uint const root) const {
   T result;
@@ -277,7 +285,7 @@ typename std::enable_if<is_registered_type<T>::value, T>::type Communicator::sca
   return result;
 }
 
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type Communicator::scatterv(
     Vector<T> const &vec, std::vector<t_int> const &sizes, t_uint const root) const {
   if (size() == 1) {
@@ -286,7 +294,8 @@ typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type Communica
     return vec;
   }
   if (rank() != root) return scatterv<T>(sizes.at(rank()), root);
-  std::vector<int> sizes_, displs;
+  std::vector<int> sizes_;
+  std::vector<int> displs;
   int i = 0;
   for (auto const size : sizes) {
     sizes_.push_back(static_cast<int>(size));
@@ -305,7 +314,7 @@ typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type Communica
   return result;
 }
 
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type Communicator::scatterv(
     t_int local_size, t_uint const root) const {
   if (rank() == root) throw std::runtime_error("Root should call the *other* scatterv");
@@ -317,7 +326,7 @@ typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type Communica
   return result;
 }
 
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<T>::value, std::vector<T>>::type
 Communicator::all_to_allv(const std::vector<T> &vec, std::vector<t_int> const &send_sizes) const {
   if (size() == 1) {
@@ -333,15 +342,29 @@ Communicator::all_to_allv(const std::vector<T> &vec, std::vector<t_int> const &s
       gather<t_int>(send_sizes[i], i);
   }
 
+  return all_to_allv<T>(vec, send_sizes, rec_sizes);
+}
+template <typename T>
+typename std::enable_if<is_registered_type<T>::value, std::vector<T>>::type
+Communicator::all_to_allv(const std::vector<T> &vec, std::vector<t_int> const &send_sizes,
+                          std::vector<t_int> const &rec_sizes) const {
+  if (size() == 1) {
+    if (send_sizes.size() == 1 and vec.size() != send_sizes.front())
+      throw std::runtime_error("Input vector size and sizes are inconsistent on root");
+    return vec;
+  }
+
   int i = 0;
-  std::vector<int> ssizes_, sdispls;
+  std::vector<int> ssizes_;
+  std::vector<int> sdispls;
   for (auto const size : send_sizes) {
     ssizes_.push_back(static_cast<int>(size));
     sdispls.push_back(i);
     i += size;
   }
   int total = 0;
-  std::vector<int> rsizes_, rdispls;
+  std::vector<int> rsizes_;
+  std::vector<int> rdispls;
   for (auto const size : rec_sizes) {
     rsizes_.push_back(static_cast<int>(size));
     rdispls.push_back(total);
@@ -352,9 +375,9 @@ Communicator::all_to_allv(const std::vector<T> &vec, std::vector<t_int> const &s
                 sdispls.data(), registered_type(T(0)), output.data(), rsizes_.data(),
                 rdispls.data(), registered_type(T(0)), **this);
   return output;
-};
+}
 
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type Communicator::all_to_allv(
     const Vector<T> &vec, std::vector<t_int> const &send_sizes) const {
   if (size() == 1) {
@@ -370,15 +393,30 @@ typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type Communica
       gather<t_int>(send_sizes[i], i);
   }
 
+  return all_to_allv<T>(vec, send_sizes, rec_sizes);
+}
+
+template <typename T>
+typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type Communicator::all_to_allv(
+    const Vector<T> &vec, std::vector<t_int> const &send_sizes,
+    std::vector<t_int> const &rec_sizes) const {
+  if (size() == 1) {
+    if (send_sizes.size() == 1 and vec.size() != send_sizes.front())
+      throw std::runtime_error("Input vector size and sizes are inconsistent on root");
+    return vec;
+  }
+
   int i = 0;
-  std::vector<int> ssizes_, sdispls;
+  std::vector<int> ssizes_;
+  std::vector<int> sdispls;
   for (auto const size : send_sizes) {
     ssizes_.push_back(static_cast<int>(size));
     sdispls.push_back(i);
     i += size;
   }
   int total = 0;
-  std::vector<int> rsizes_, rdispls;
+  std::vector<int> rsizes_;
+  std::vector<int> rdispls;
   for (auto const size : rec_sizes) {
     rsizes_.push_back(static_cast<int>(size));
     rdispls.push_back(total);
@@ -389,9 +427,9 @@ typename std::enable_if<is_registered_type<T>::value, Vector<T>>::type Communica
                 sdispls.data(), registered_type(T(0)), output.data(), rsizes_.data(),
                 rdispls.data(), registered_type(T(0)), **this);
   return output;
-};
+}
 
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<T>::value, std::vector<T>>::type Communicator::gather(
     T const value, t_uint const root) const {
   assert(root < size());
@@ -407,13 +445,13 @@ typename std::enable_if<is_registered_type<T>::value, std::vector<T>>::type Comm
   return result;
 }
 
-template <class CONTAINER, class T>
+template <typename CONTAINER, typename T>
 CONTAINER Communicator::gather_(CONTAINER const &vec, std::vector<t_int> const &sizes,
                                 t_uint const root) const {
   assert(root < size());
   if (sizes.size() != size() and rank() == root)
     throw std::runtime_error("Sizes and communicator size do not match on root");
-  else if (rank() != root and sizes.size() != 0 and sizes.size() != size())
+  else if (rank() != root and !sizes.empty() and sizes.size() != size())
     throw std::runtime_error(
         "Outside root, sizes should be either empty or match the number of procs");
   else if (sizes.size() == size() and sizes[rank()] != static_cast<t_int>(vec.size()))
@@ -423,7 +461,8 @@ CONTAINER Communicator::gather_(CONTAINER const &vec, std::vector<t_int> const &
 
   if (rank() != root) return gather_<CONTAINER, T>(vec, root);
 
-  std::vector<int> sizes_, displs;
+  std::vector<int> sizes_;
+  std::vector<int> displs;
   int result_size = 0;
   for (auto const size : sizes) {
     sizes_.push_back(static_cast<int>(size));
@@ -438,7 +477,7 @@ CONTAINER Communicator::gather_(CONTAINER const &vec, std::vector<t_int> const &
   return result;
 }
 
-template <class CONTAINER, class T>
+template <typename CONTAINER, typename T>
 CONTAINER Communicator::gather_(CONTAINER const &vec, t_uint const root) const {
   assert(root < size());
   if (rank() == root) throw std::runtime_error("Root should call the *other* gather");
@@ -448,7 +487,7 @@ CONTAINER Communicator::gather_(CONTAINER const &vec, t_uint const root) const {
   return CONTAINER();
 }
 
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<T>::value, std::set<T>>::type Communicator::gather(
     std::set<T> const &set, std::vector<t_int> const &sizes, t_uint const root) const {
   assert(root < size());
@@ -465,7 +504,7 @@ typename std::enable_if<is_registered_type<T>::value, std::set<T>>::type Communi
   return std::set<T>(buffer.data(), buffer.data() + buffer.size());
 }
 
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<T>::value, std::set<T>>::type Communicator::gather(
     std::set<T> const &set, t_uint const root) const {
   assert(root < size());
@@ -477,7 +516,7 @@ typename std::enable_if<is_registered_type<T>::value, std::set<T>>::type Communi
   return std::set<T>();
 }
 
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<T>::value, T>::type Communicator::broadcast(
     T const &value, t_uint const root) const {
   assert(root < size());
@@ -488,7 +527,7 @@ typename std::enable_if<is_registered_type<T>::value, T>::type Communicator::bro
   return result;
 }
 //! Receive broadcast object
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<T>::value, T>::type Communicator::broadcast(
     t_uint const root) const {
   assert(root < size());
@@ -499,7 +538,7 @@ typename std::enable_if<is_registered_type<T>::value, T>::type Communicator::bro
   return result;
 }
 
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<typename T::Scalar>::value, T>::type
 Communicator::broadcast(T const &vec, t_uint const root) const {
   if (size() == 1) return vec;
@@ -512,7 +551,7 @@ Communicator::broadcast(T const &vec, t_uint const root) const {
             root, **this);
   return vec;
 }
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<typename T::Scalar>::value, T>::type
 Communicator::broadcast(t_uint const root) const {
   assert(root < size());
@@ -524,7 +563,7 @@ Communicator::broadcast(t_uint const root) const {
   MPI_Bcast(result.data(), result.size(), Type<typename T::Scalar>::value, root, **this);
   return result;
 }
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<typename T::value_type>::value and
                             not std::is_base_of<Eigen::EigenBase<T>, T>::value,
                         T>::type
@@ -538,7 +577,7 @@ Communicator::broadcast(T const &vec, t_uint const root) const {
             Type<typename T::value_type>::value, root, **this);
   return vec;
 }
-template <class T>
+template <typename T>
 typename std::enable_if<is_registered_type<typename T::value_type>::value and
                             not std::is_base_of<Eigen::EigenBase<T>, T>::value,
                         T>::type
@@ -551,7 +590,6 @@ Communicator::broadcast(t_uint const root) const {
   MPI_Bcast(result.data(), result.size(), Type<typename T::value_type>::value, root, **this);
   return result;
 }
-}  // namespace mpi
-}  // namespace sopt
+} // namespace sopt::mpi
 #endif /* ifdef SOPT_MPI */
 #endif /* ifndef SOPT_MPI_COMMUNICATOR */
