@@ -2,9 +2,9 @@
 #include <iterator>
 #include <regex>
 #include <vector>
-#include "catch.hpp"
-#include <sopt/logging.h>
-#include <sopt/mpi/session.h>
+#include "catch2/catch_all.hpp"
+#include "sopt/logging.h"
+#include "sopt/mpi/session.h"
 using namespace sopt;
 
 std::vector<char const *> cargs;
@@ -25,7 +25,7 @@ int main(int argc, const char **argv) {
   // Removes output qualifiers from arguments
   auto avoid = false;
   auto const N = std::string("--out=").size();
-  std::vector<std::string> args(argv, argv + argc);
+  std::vector<std::string> const args(argv, argv + argc);
   for (auto const &arg : args)
     if (avoid)
       avoid = false;
@@ -38,8 +38,6 @@ int main(int argc, const char **argv) {
   if (returnCode != 0)  // Indicates a command line error
     return returnCode;
 
-  sopt::logging::initialize();
-
   auto const result = session.run();
   return result;
 }
