@@ -7,26 +7,26 @@
 
 namespace sopt {
 // Wraps calls to sampling and wavelets to C style
-template <class T>
+template <typename T>
 struct CData {
-  typedef Eigen::Matrix<T, Eigen::Dynamic, 1> t_Vector;
+  using t_Vector = Eigen::Matrix<T, Eigen::Dynamic, 1>;
   typename t_Vector::Index nin, nout;
   sopt::LinearTransform<t_Vector> const &transform;
   t_uint direct_calls, adjoint_calls;
 };
 
-template <class T>
+template <typename T>
 void direct_transform(void *out, void *in, void **data) {
   CData<T> const &cdata = *(CData<T> *)data;
-  typedef Eigen::Matrix<T, Eigen::Dynamic, 1> t_Vector;
+  using t_Vector = Eigen::Matrix<T, Eigen::Dynamic, 1>;
   t_Vector const eval = cdata.transform * t_Vector::Map((T *)in, cdata.nin);
   ++(((CData<T> *)data)->direct_calls);
   t_Vector::Map((T *)out, cdata.nout) = eval;
 }
-template <class T>
+template <typename T>
 void adjoint_transform(void *out, void *in, void **data) {
   CData<T> const &cdata = *(CData<T> *)data;
-  typedef Eigen::Matrix<T, Eigen::Dynamic, 1> t_Vector;
+  using t_Vector = Eigen::Matrix<T, Eigen::Dynamic, 1>;
   t_Vector const eval = cdata.transform.adjoint() * t_Vector::Map((T *)in, cdata.nout);
   ++(((CData<T> *)data)->adjoint_calls);
   t_Vector::Map((T *)out, cdata.nin) = eval;
